@@ -11,7 +11,7 @@ use JSON::XS qw(decode_json encode_json);
 use Time::HiRes qw(gettimeofday time);
 use Data::Dumper;
 
-our $VERSION = "0.040";
+our $VERSION = "0.041";
 
 my %INFO_ROUTES = (
     version => '/version',
@@ -39,8 +39,24 @@ sub new {
             sprintf 'http://%s:%d', @params{qw<host port>};
     }
 
+    my $http_tiny = HTTP::Tiny->new();
+
+    my $strict_mode = 0;
+    if ($params{strict_mode}) {
+        if ($params{strict_mode} == 1) {
+            $strict_mode = $params{strict_mode};
+        }
+        else {
+            die "Invalid value. Strict mode can have only one of the following values: [undef, 0, 1]";    
+        }
+    }
+
     # we only care about the service, and we assured it exists
-    return bless { service => $params{'service'} }, $class;
+    return bless {
+        service     => $params{'service'},
+        http_tiny   => $http_tiny,
+        strict_mode => $strict_mode,
+    }, $class;
 }
 
 
@@ -59,7 +75,7 @@ sub list {
 
     my $uri = $self->{service} . $INFO_ROUTES{list};
 
-    my $http = HTTP::Tiny->new();
+    my $http = $self->{http_tiny} ||=  HTTP::Tiny->new();
 
     my $rs = $http->get($uri, {
          headers => {
@@ -68,8 +84,15 @@ sub list {
     });
 
     if ( ! $rs->{success} ) {
-       $self->{last_error} = _extract_error_tiny($uri, $rs);
-       return undef;
+
+        $self->{last_error} = _extract_error_tiny($uri, $rs);
+
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return undef;
+        }
     }
 
     my $response = decode_json($rs->{content});
@@ -94,7 +117,7 @@ sub version {
 
     my $uri = $self->{service} . $INFO_ROUTES{version};
 
-    my $http = HTTP::Tiny->new();
+    my $http = $self->{http_tiny} ||= HTTP::Tiny->new();
 
     my $rs = $http->get($uri, {
          headers => {
@@ -103,8 +126,15 @@ sub version {
     });
 
     if ( ! $rs->{success} ) {
-       $self->{last_error} = _extract_error_tiny($uri, $rs);
-       return undef;
+
+        $self->{last_error} = _extract_error_tiny($uri, $rs);
+
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return undef;
+        }
     }
 
     my $response = decode_json($rs->{content});
@@ -160,8 +190,8 @@ sub _get_tiny {
 
     my $url = $self->{service} . $uri;
 
-    my $http = HTTP::Tiny->new();
-
+    my $http = $self->{http_tiny} ||= HTTP::Tiny->new();
+    
     my $rs = $http->get($url, {
         headers => {
             'Accept' => 'application/json',
@@ -169,8 +199,15 @@ sub _get_tiny {
     });
 
     if ( ! $rs->{success} ) {
-       $self->{last_error} = _extract_error_tiny($url, $rs);
-       return undef;
+
+        $self->{last_error} = _extract_error_tiny($url, $rs);
+
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return undef;
+        }
     }
 
     my $response = decode_json($rs->{content});
@@ -246,7 +283,7 @@ sub _multiget_tiny {
 
     my $url = $self->{service} . $uri;
 
-    my $http = HTTP::Tiny->new();
+    my $http = $self->{http_tiny} ||= HTTP::Tiny->new();
 
     my $data_format = 'application/json';
 
@@ -257,8 +294,15 @@ sub _multiget_tiny {
     });
 
     if ( ! $rs->{success} ) {
-       $self->{last_error} = _extract_error_tiny($url, $rs);
-       return undef;
+
+        $self->{last_error} = _extract_error_tiny($url, $rs);
+
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return undef;
+        }
     }
 
     my $response = decode_json($rs->{content});
@@ -400,7 +444,7 @@ sub put {
     my $route = '/' . uri_escape($table) . '/false-row-key';
     my $uri = $self->{service} . $route;
 
-    my $http = HTTP::Tiny->new();
+    my $http = $self->{http_tiny} ||= HTTP::Tiny->new();
 
     my $rs = $http->request('PUT', $uri, {
         content => $JSON_Command,
@@ -411,8 +455,15 @@ sub put {
     });
 
     if ( ! $rs->{success} ) {
-       $self->{last_error} = _extract_error_tiny($uri, $rs);
-       return undef;
+
+        $self->{last_error} = _extract_error_tiny($uri, $rs);
+        
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return undef;
+        }
     }
 
 }
@@ -447,7 +498,7 @@ sub delete {
         $route .= sprintf(":%s", $column) if($column);
     }
 
-    $http = HTTP::Tiny->new();
+    $http = $self->{http_tiny} ||= HTTP::Tiny->new();
     $url = sprintf("%s%s", $self->{service}, $route);
     $rs = $http->delete($url, {
         headers => {
@@ -460,8 +511,15 @@ sub delete {
        return 1;
     }
     else {
+
         $self->{last_error} = _extract_error_tiny($url, $rs);
-        return 0;
+        
+        if ($self->{strict_mode}) {
+            die "request error: " . Dumper($self->{last_error});
+        }
+        else {
+            return 0;
+        }
     }
 }
 
@@ -513,7 +571,7 @@ sub _build_get_uri {
                 }
                 else {
                     push @get_urls, { url => $current_url, len => length($current_url) };
-                    $current_url = undef;
+                    $current_url = $uri . "/" . uri_escape($column);
                 }
             }
         }
@@ -542,8 +600,7 @@ sub _build_multiget_uri {
     my $keys  = $query->{where}->{key_in};
     my $table = $query->{table};
 
-    my $route_base = '/' . $table . '/multiget?';
-    my $uri_base = $route_base;
+    my $uri_base = '/' . $table . '/multiget?';
 
     my @multiget_urls = ();
     my $current_url = undef;
@@ -558,7 +615,7 @@ sub _build_multiget_uri {
             }
             else {
                 push @multiget_urls, { url => $current_url, len => length($current_url) };
-                $current_url = undef;
+                $current_url = $uri_base . "row=" . uri_escape($key);
             }
         }
     }
@@ -591,7 +648,8 @@ sub _extract_error_tiny {
          return {
             type => 'Unkown',
             info => 'No status in response',
-            uri => $uri
+            uri  => $uri,
+            http_response => $res,
         };
     }
 
@@ -895,7 +953,7 @@ Information on error is stored in hbase object under key last error:
 
 =head1 VERSION
 
-Current version: 0.040
+Current version: 0.041
 
 =head1 AUTHOR
 
@@ -913,6 +971,8 @@ Robert Nilsson, <rn@orbstation.com>
 
 tsheasha - T Sheasha, <tarek.sheasha@gmail.com>
 
+dtcyganov - Dmitrii Tcyganov, C<< <dtcyganov at github.com> >>
+
 =head1 COPYRIGHT
 
 Copyright (c) 2014 the HBase::JSONRest L</AUTHOR> and L</CONTRIBUTORS>
